/**************************
  FIELD.C

    A collection of routines to allow easy management of a
    collection of user editable data fields.  

    Written by Aaron C. Caba    1/92
    Compiled with Turbo C++ 1.0

History:

 Version 1.5:
  03/10/92 ACC Move #includes to FIELD.H

 Version 1.4:
  02/11/92 ACC Major rearangement of all code:
                 move all field manipulation functions to this file
                 clean up header files so only necessary stuff is in each
                 throw in some comments

 *************************/

#include "field.h"      /* function prototypes          */

/* ================================================================ */

/* array of field_type to hold information about each field */
extern struct field_struct field[];
extern char *field_default[];
extern int parm_field;


/* ********************************
    Function:   fieldinput
    Description:  edit a particular field until a
                  control character is pressed
    Arguments:  fieldnum -- number of field to edit
                data -- address to place user data at
                msg -- informational message to print with disp_message()
                dy -- offset displayed field down this many rows
    Return Value:  what control character was pressed
*/
int fieldinput(int fieldnum, char *data, char *msg, int dy)
{
    int done=FALSE,chcount,
        retvalue,
        first_time=TRUE;

    char ch;

    chcount=0;
    disp_field(fieldnum,data,EGA_WHITE,dy);
    while (!done) {
        ch=getch();
        retvalue=-1;
        disp_message(msg);
        switch(field[fieldnum].type) {
            case ALPHA:
                if (isprint(ch)) {
                    if (first_time) {
                        strcpy(data,"");
                        first_time=FALSE;
                    }
                    if (chcount < field[fieldnum].len) {
                        strncat(data,&ch,1);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                        chcount++;
                    }
                }
                else
                    if (iscntrl(ch)) {
                        retvalue=control_char(ch, data, &chcount);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                    }
                    else disp_message("Please press: <A-Z | 0-9>");
                break;
            case INTEGER:
                if (isdigit(ch)) {
                    if (first_time) {
                        strcpy(data,"");
                        first_time=FALSE;
                    }
                    if (chcount < field[fieldnum].len) {
                        strncat(data,&ch,1);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                        chcount++;
                    }
                }
                else
                    if (iscntrl(ch)) {
                        retvalue=control_char(ch, data, &chcount);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                    }
                    else disp_message("Please press: <0-9>");
                break;
            case DECIMAL:
                if (isdigit(ch) || ch=='.') {
                    if (first_time) {
                        strcpy(data,"");
                        first_time=FALSE;
                    }
                    if (chcount < field[fieldnum].len) {
                        strncat(data,&ch,1);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                        chcount++;
                    }
                }
                else
                    if (iscntrl(ch)) {
                        retvalue=control_char(ch, data, &chcount);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                    }
                    else disp_message("Please press: <0-9 || .>");
                break;
            case XYZ:
                if (isalpha(ch)){
                    switch(ch) {
                        case 'X':
                        case 'x':
                        case 'Y':
                        case 'y':
                        case 'Z':
                        case 'z':
                            if (first_time) {
                                strcpy(data,"");
                                first_time=FALSE;
                            }
                            if (chcount < field[fieldnum].len) {
                                strncat(data,&ch,1);
                                disp_field(fieldnum,data,EGA_WHITE,dy);
                                chcount++;
                            }
                            break;
                        default: disp_message("Please press: <X,Y,Z>");
                                 break;
                    }
                }
                else if (iscntrl(ch)) {
                    retvalue=control_char(ch, data, &chcount);
                    disp_field(fieldnum,data,EGA_WHITE,dy);
                }
                else disp_message("Please press: <X,Y,Z>");
                break;
            case SIGNED_INTEGER:
                disp_field(fieldnum, data, EGA_WHITE, dy);
                if (isdigit(ch) || ch=='.' || ch=='-') {
                    if (first_time) {
                        strcpy(data,"");
                        first_time=FALSE;
                    }
                    if (chcount < field[fieldnum].len) {
                        strncat(data,&ch,1);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                        chcount++;
                    }
                }
                else
                    if (iscntrl(ch)) {
                        retvalue=control_char(ch, data, &chcount);
                        disp_field(fieldnum,data,EGA_WHITE,dy);
                    }
                    else disp_message("Please press: <0-9 || - >");
               break;
            default:
				error_fn(EUNK,"fieldinput");
                break;
        }  /* end switch(field[].type)  */

        /* determine if done editing this field */
        switch(retvalue) {
            case BKSPC: done=FALSE; break;
            case ESC:
            case UP:
            case DOWN:
            case PGUP:
            case PGDN:
            case F10:
            case RETURN: done=TRUE; break;
        }

    }  /* end while(!done) */
    if (strlen(data)==0) strcpy(data,field_default[fieldnum]);
    disp_field(fieldnum,data,EGA_DARKGRAY,dy);
    return(retvalue);
}  /* end fieldinput */


/* ********************************
    Function:   control_char
    Description:  determine what control character was pressed,
                  if it was BKSPC, delete last character from 'data'
                  and decrement 'chcount'.
                  Then return what the control character was.
    Arguments:  ch -- incoming control character
                data -- string containing current field contents
                chcount -- number of characters in the field
    Return Value:   what control character was pressed
*/
char control_char(char ch, char *data, int *chcount)
{
    char temp[MAX_FIELD_LEN];
    int retvalue=-1;

    strcpy(temp,"");
    switch(ch) {
        case RETURN: retvalue=RETURN;
                     break;
        case BKSPC: retvalue=BKSPC;
                    (*chcount)--;
                    if (*chcount < 0) *chcount=0;
                    if (*chcount) {
                        strncpy(temp,data,*chcount);
                        temp[*chcount]='\0';
                    }
                    strcpy(data,temp);
                    break;
        case ESC:  retvalue=ESC;
                   break;
        default:
            ch=getch();
            switch(ch) {
                case UP:    retvalue=UP;   break;
                case DOWN:  retvalue=DOWN; break;
                case PGUP:  retvalue=PGUP; break;
                case PGDN:  retvalue=PGDN; break;
                case F10:   retvalue=F10;  break;
                default:    retvalue=DOWN; break;
            }  /* end switch */
            break;
    } /* end switch(ch) */
    return(retvalue);
} /* end control_char */


void disp_field(int n, char *data, int color, int dy)
{
    int p=0;

    if (parm_field==TRUE) p=15;
    erase_text(field[n].x+p, field[n].y+dy, field[n].len);
    disp_text(field[n].x+p, field[n].y+dy, data, color);
}


/* ********************************
    Function:   verify
    Description:  print "msg"+"(Y or N)?" and query for a
                  yes or no answer
    Arguments:  msg -- string to print before the '(Y or N)?'
    Return Value:  TRUE if 'yes' answer
                   FALSE if 'no' answer
*/
int verify(char *msg)
{
    char ch, str[70];
    int done=FALSE, retval;

    strcpy(str,msg);
	strcat(str,"(Y or N)?");
    disp_message(str);
    while (kbhit()) getch();
    while (!done) {
        ch=getch();
        if (ch == 'Y' || ch =='y') {
            retval=TRUE;
            done=TRUE;
        } else if (ch == 'N' || ch == 'n') {
            retval=FALSE;
            done = TRUE;
        }
    }
    return(retval);
} /* end verify */


/* ********************************
    Function:   assign_default
    Description:  change all data fields to their default values
    Arguments:  data -- the list of data fields
    Return Value:  none
*/
void assign_default(char *data[])
{
    int i;

    for(i=0; i < MAXFIELDS; i++) strcpy(data[i],field_default[i]);
}


/* ********************************
    Function:   disp_message
    Description:  display a message at MESSAGE_Y_POS on the screen
    Arguments:  msg -- string to display
    Return Value:  none
*/
void disp_message(char *msg)
{
    int maxlen=70,
        x,
        ypos=MESSAGE_Y_POS;

    if (getgraphmode() == VGAHI) ypos += 13;
    if (strlen(msg) > maxlen) strncat(msg,msg,maxlen); /* TRUNCATE IF NEEDED */
    x=(80-maxlen)/2;
    erase_text(x,ypos,maxlen);
    x=(80-strlen(msg))/2;
    disp_text(x,ypos,msg,EGA_DARKGRAY);
}


/* ********************************
    Function:   disp_text
    Description:  To display a given string 'msg' on the screen at location
                  (x,y) in the given colors.
    Arguments:  x -- the starting x coordinate of the message
                y -- y coordinate of the message
                msg -- what you want printed
                color -- foreground color to print in
    Return Value:  none
*/
void disp_text(int x, int y, char *msg, int color)
{
    setcolor(color);
    outtextxy(x*CHAR_BOX_X, y*CHAR_BOX_Y+2, msg);
}


/* ********************************
    Function:   erase_text
    Description:  erase text starting at (x,y) and for 'len' characters.
    Arguments:  x, y -- coordinates to start erasing
                n  -- number of characters to erase
    Return Value:  none
*/
void erase_text(int x, int y, int n)
{
    setfillstyle(SOLID_FILL,EGA_LIGHTGRAY);
    x*=CHAR_BOX_X;
    y*=CHAR_BOX_Y;
    bar(x, y, x+n*CHAR_BOX_X, y+CHAR_BOX_Y-1);
}

